@@ -16,20 +16,19 @@ pois.envMU <- function(X, Y, u, initial = NULL){
   fit <- stats::glm(Y ~ X, family = stats::poisson)
   mu <- as.vector(fit$coefficients[1])
   beta <- as.vector(fit$coefficients[2 : (p + 1)])
-  theta <- matrix(1, n, 1) %*% mu + X %*% beta
+  theta <- mu + X %*% beta
   c.theta <- - exp(theta)
   c.theta.mean <- mean(c.theta)
   weight <- c.theta / c.theta.mean
   V <- theta + ((Y - c.theta) / weight)
-  W <- diag(as.vector(weight))
-  wx <- W %*% X
-  mean.wx <- apply(wx, 2, mean)
-  wxx <- X - matrix(1, nrow = n) %*% mean.wx
-  sigwx <- crossprod(wxx, W) %*% wxx / n
-  wv <- W %*% V
+  wx <- sweep(X, 1, weight, '*')
+  mean.wx <- colMeans(wx)
+  wxx <- sweep(X, 2, mean.wx, '-')
+  sigwx <- crossprod(wxx, sweep(wxx, 1, weight, '*')) / n
+  wv <- weight * V
   mean.wv <- mean(wv)
-  wvv <- V - matrix(1, nrow = n) %*% mean.wv
-  sigwxv <- crossprod(wxx, W) %*% wvv / n
+  wvv <- as.matrix(V - mean.wv)
+  sigwxv <- crossprod(wxx, sweep(wvv, 1, weight, '*')) / n
   inv.sigwx <- chol2inv(chol(sigwx))
 
   M.init <- inv.sigwx / (- c.theta.mean)
@@ -44,7 +43,7 @@ pois.envMU <- function(X, Y, u, initial = NULL){
 
   if (u == 0) {
     Gammahat <- NULL
-    Gamma0hat <- diag(r)
+    Gamma0hat <- diag(p)
     tmp.M <- eigen(sigX)
     mu <- log(mean(Y))
     muh <- matrix(1, n, 1) %*% mu
@@ -55,7 +54,7 @@ pois.envMU <- function(X, Y, u, initial = NULL){
     V <- Y
     objfun <- Cn1 - n/2 * sum(log(tmp.M$values))
   } else if (u == p) {
-    Gammahat <- diag(r)
+    Gammahat <- diag(p)
     Gamma0hat <- NULL
     tmp.M <- eigen(sigX)
     eta <- beta
@@ -63,8 +62,8 @@ pois.envMU <- function(X, Y, u, initial = NULL){
     Cn1 <- t(Y) %*% theta - colSums(exp(theta))
     objfun <- Cn1 - n/2 * sum(log(tmp.M$values))
   } else if (u == p - 1) {
-    maxiter = 10000
-    ftol = 0.0001
+    maxiter = 1000
+    ftol = 0.001
     
     M <- sigX
     MU <- sigX
@@ -77,14 +76,14 @@ pois.envMU <- function(X, Y, u, initial = NULL){
     e3 <- eigen(sigX)
     temp1 <- sum(log(e1$values)) + sum(log(e2$values)) + sum(log(e3$values))
     obj1 <- Cn1 - (n / 2) * temp1
-
-    GiX <- X %*% gamma.init
-    fit <- stats::glm(Y ~ GiX, family = stats::poisson)
-    eta <- as.matrix(fit$coefficients[2 : (u + 1)])
   
     init <- gamma.init
     GEidx <- GE(init)
     Ginit <- init %*% solve(init[GEidx[1:u], ])
+    GiX <- X %*% Ginit
+    fit <- stats::glm(Y ~ GiX, family = stats::poisson)
+    eta <- as.matrix(fit$coefficients[2 : (u + 1)])
+    
     j <- GEidx[p]
     g <- as.matrix(Ginit[j, ])
     g1 <- Ginit[-j, ]
@@ -109,6 +108,10 @@ pois.envMU <- function(X, Y, u, initial = NULL){
       fobj <- function(x) {
         tmp2 <- x + t2
         tmp3 <- x + t3
+        Ginit[j, ] <- x
+        GiX <- X %*% Ginit
+        fit <- stats::glm(Y ~ GiX, family = stats::poisson)
+        eta <- as.matrix(fit$coefficients[2 : (u + 1)])
         tmp4 <- X1 %*% t(x) %*% eta + t6
         T2 <- invC1 %*% tmp2
         T3 <- invC2 %*% tmp3 
@@ -121,6 +124,10 @@ pois.envMU <- function(X, Y, u, initial = NULL){
       gobj <- function(x) {
         tmp2 <- x + t2
         tmp3 <- x + t3
+        Ginit[j, ] <- x
+        GiX <- X %*% Ginit
+        fit <- stats::glm(Y ~ GiX, family = stats::poisson)
+        eta <- as.matrix(fit$coefficients[2 : (u + 1)])
         tmp4 <- X1 %*% t(x) %*% eta + t6
         Cn <- Y - exp(tmp4)
         T2 <- invC1 %*% tmp2
@@ -150,28 +157,27 @@ pois.envMU <- function(X, Y, u, initial = NULL){
         mu <- as.vector(fit$coefficients[1])
         eta <- as.matrix(fit$coefficients[2 : (u + 1)])
         beta <- Ginit %*% eta
-        theta <- matrix(1, n, 1) %*% mu + X %*% beta
+        theta <- mu + X %*% beta
         c.theta <- - exp(theta)
         c.theta.mean <- mean(c.theta)
         weight <- c.theta / c.theta.mean
         V <- theta + ((Y - c.theta) / weight)
-        W <- diag(as.vector(weight))
-        wx <- W %*% X
-        mean.wx <- apply(wx, 2, mean)
-        wxx <- X - matrix(1, nrow = n) %*% mean.wx
-        sigwx <- crossprod(wxx, W) %*% wxx / n
-        wv <- W %*% V
+        wx <- sweep(X, 1, weight, '*')
+        mean.wx <- colMeans(wx)
+        wxx <- sweep(X, 2, mean.wx, '-')
+        sigwx <- crossprod(wxx, sweep(wxx, 1, weight, '*')) / n
+        wv <- weight * V
         mean.wv <- mean(wv)
-        wvv <- V - matrix(1, nrow = n) %*% mean.wv
-        sigwxv <- crossprod(wxx, W) %*% wvv / n
+        wvv <- as.matrix(V - mean.wv)
+        sigwxv <- crossprod(wxx, sweep(wvv, 1, weight, '*')) / n
 
       
         e1 <- eigen(t(Ginit) %*% M %*% Ginit)
         e2 <- eigen(t(Ginit) %*% invMU %*% Ginit)
         e3 <- eigen(crossprod(Ginit))
-        e4 <- matrix(1, n, 1) %*% mu + X %*% beta
+        e4 <- mu + X %*% beta
         e5 <- crossprod(Y, e4) - colSums(exp(e4))
-        obj2 <- - n/2 * (sum(log(e1$values)) + sum(log(e2$values)) + sum(log(e3$values))) + e5
+        obj2 <- - n/2 * (sum(log(e1$values)) + sum(log(e2$values))) + sum(log(e3$values)) + e5
         if (abs(obj1 - obj2) < ftol) {
           break
         }
@@ -187,32 +193,30 @@ pois.envMU <- function(X, Y, u, initial = NULL){
       mu <- as.vector(fit$coefficients[1])
       eta <- matrix(fit$coefficients[2 : (u + 1)])
       beta <- Gammahat %*% eta
-      theta <- matrix(1, n, 1) %*% mu + X %*% beta
-      theta <- matrix(1, n, 1) %*% mu + X %*% beta
+      
+      theta <- mu + X %*% beta
       c.theta <- - exp(theta)
       c.theta.mean <- mean(c.theta)
       weight <- c.theta / c.theta.mean
-      V <- theta + ((Y - c.theta) / weight)
-      W <- diag(as.vector(weight))
-      wx <- W %*% X
-      mean.wx <- apply(wx, 2, mean)
-      wxx <- X - matrix(1, nrow = n) %*% mean.wx
-      sigwx <- crossprod(wxx, W) %*% wxx / n
+      wx <- sweep(X, 1, weight, '*')
+      mean.wx <- colMeans(wx)
+      wxx <- sweep(X, 2, mean.wx, '-')
+      sigwx <- crossprod(wxx, sweep(wxx, 1, weight, '*')) / n
       inv.sigwx <- chol2inv(chol(sigwx))
       var <- inv.sigwx / (- c.theta.mean)
       e1 <- eigen(t(Gammahat) %*% M %*% Gammahat)
       e2 <- eigen(t(Gammahat) %*% invMU %*% Gammahat)
       e3 <- eigen(M)
-      e4 <- matrix(1, n, 1) %*% mu + X %*% Gammahat %*% eta
+      e4 <- mu + X %*% Gammahat %*% eta
       e5 <- crossprod(Y, e4) - colSums(exp(e4))
       Gamma0hat <- qr.Q(a, complete = TRUE)[, (u + 1):r, drop = FALSE]
-      objfun <- - n/2 * (sum(log(e1$values)) + sum(log(e2$values)) + sum(log(e3$values))) + e5
+      objfun <- - n/2 * (sum(log(e1$values)) + sum(log(e2$values))) + sum(log(e3$values)) + e5
       Gammahat <- as.matrix(Gammahat)
       Gamma0hat <- as.matrix(Gamma0hat)
   
     } else {
-      maxiter = 10000
-      ftol = 0.0001
+      maxiter = 100
+      ftol = 0.001
       
       M <- sigX
       MU <- sigX
@@ -226,13 +230,12 @@ pois.envMU <- function(X, Y, u, initial = NULL){
       temp1 <- sum(log(e1$values)) + sum(log(e2$values)) + sum(log(e3$values))
       obj1 <- Cn1 - (n / 2) * temp1
     
-      GiX <- X %*% gamma.init
-      fit <- stats::glm(Y ~ GiX, family = stats::poisson)
-      eta <- as.matrix(fit$coefficients[2 : (u + 1)])
-    
       init <- gamma.init
       GEidx <- GE(init)
       Ginit <- init %*% solve(init[GEidx[1:u], ])
+      GiX <- X %*% Ginit
+      fit <- stats::glm(Y ~ GiX, family = stats::poisson)
+      eta <- as.matrix(fit$coefficients[2 : (u + 1)])
       GUG <- crossprod(Ginit, (M %*% Ginit))
       GVG <- crossprod(Ginit, (invMU %*% Ginit))
       t4 <- crossprod(Ginit[GEidx[(u + 1):p], ], Ginit[GEidx[(u + 1):p], ]) + diag(u)
@@ -255,7 +258,7 @@ pois.envMU <- function(X, Y, u, initial = NULL){
           X1 <- X[ , j]
           X2 <- X[ , -j]
           t5 <- X1 %*% t(g) %*% eta
-          t6 <- matrix(1, n, 1) %*% mu + X2 %*% g1 %*% eta
+          t6 <- mu + X2 %*% g1 %*% eta
           t7 <- t5 + t6
           t8 <- t(Y) %*% t7
           et7 <- exp(t7)
@@ -264,6 +267,10 @@ pois.envMU <- function(X, Y, u, initial = NULL){
           fobj <- function(x) {
             tmp2 <- x + t2
             tmp3 <- x + t3
+            Ginit[j, ] <- x
+            GiX <- X %*% Ginit
+            fit <- stats::glm(Y ~ GiX, family = stats::poisson)
+            eta <- as.matrix(fit$coefficients[2 : (u + 1)])
             tmp4 <- X1 %*% t(x) %*% eta + t6
             T1 <- invt4 %*% x
             T2 <- invC1 %*% tmp2
@@ -277,6 +284,10 @@ pois.envMU <- function(X, Y, u, initial = NULL){
           gobj <- function(x) {
             tmp2 <- x + t2
             tmp3 <- x + t3
+            Ginit[j, ] <- x
+            GiX <- X %*% Ginit
+            fit <- stats::glm(Y ~ GiX, family = stats::poisson)
+            eta <- as.matrix(fit$coefficients[2 : (u + 1)])
             tmp4 <- X1 %*% t(x) %*% eta + t6
             Cn <- Y - exp(tmp4)
             T1 <- invt4 %*% x
@@ -311,20 +322,20 @@ pois.envMU <- function(X, Y, u, initial = NULL){
         mu <- as.vector(fit$coefficients[1])
         eta <- as.matrix(fit$coefficients[2 : (u + 1)])
         beta <- Ginit %*% eta
-        theta <- matrix(1, n, 1) %*% mu + X %*% beta
+        theta <- mu + X %*% beta
         c.theta <- - exp(theta)
         c.theta.mean <- mean(c.theta)
         weight <- c.theta / c.theta.mean
         V <- theta + ((Y - c.theta) / weight)
-        W <- diag(as.vector(weight))
-        wx <- W %*% X
-        mean.wx <- apply(wx, 2, mean)
-        wxx <- X - matrix(1, nrow = n) %*% mean.wx
-        sigwx <- crossprod(wxx, W) %*% wxx / n
-        wv <- W %*% V
+        wx <- sweep(X, 1, weight, '*')
+        mean.wx <- colMeans(wx)
+        wxx <- sweep(X, 2, mean.wx, '-')
+        sigwx <- crossprod(wxx, sweep(wxx, 1, weight, '*')) / n
+        wv <- weight * V
         mean.wv <- mean(wv)
-        wvv <- V - matrix(1, nrow = n) %*% mean.wv
-        sigwxv <- crossprod(wxx, W) %*% wvv / n
+        wvv <- as.matrix(V - mean.wv)
+        sigwxv <- crossprod(wxx, sweep(wvv, 1, weight, '*')) / n
+
 
       
         e1 <- eigen(t(Ginit) %*% M %*% Ginit)
@@ -332,7 +343,7 @@ pois.envMU <- function(X, Y, u, initial = NULL){
         e3 <- eigen(crossprod(Ginit))
         e4 <- matrix(1, n, 1) %*% mu + X %*% beta
         e5 <- crossprod(Y, e4) - colSums(exp(e4))
-        obj2 <- - n/2 * (sum(log(e1$values)) + sum(log(e2$values)) + sum(log(e3$values))) + e5
+        obj2 <- - n/2 * (sum(log(e1$values)) + sum(log(e2$values))) + sum(log(e3$values)) + e5
         if (abs(obj1 - obj2) < ftol) {
           break
         }
@@ -352,22 +363,20 @@ pois.envMU <- function(X, Y, u, initial = NULL){
       c.theta <- - exp(theta)
       c.theta.mean <- mean(c.theta)
       weight <- c.theta / c.theta.mean
-      V <- theta + ((Y - c.theta) / weight)
-      W <- diag(as.vector(weight))
-      wx <- W %*% X
-      mean.wx <- apply(wx, 2, mean)
-      wxx <- X - matrix(1, nrow = n) %*% mean.wx
-      sigwx <- crossprod(wxx, W) %*% wxx / n
-      
+      wx <- sweep(X, 1, weight, '*')
+      mean.wx <- colMeans(wx)
+      wxx <- sweep(X, 2, mean.wx, '-')
+      sigwx <- crossprod(wxx, sweep(wxx, 1, weight, '*')) / n
       inv.sigwx <- chol2inv(chol(sigwx))
+      
       var <- inv.sigwx / (- c.theta.mean)
       e1 <- eigen(t(Gammahat) %*% M %*% Gammahat)
       e2 <- eigen(t(Gammahat) %*% invMU %*% Gammahat)
       e3 <- eigen(M)
-      e4 <- matrix(1, n, 1) %*% mu + X %*% Gammahat %*% eta
+      e4 <- mu + X %*% Gammahat %*% eta
       e5 <- crossprod(Y, e4) - colSums(exp(e4))
       Gamma0hat <- qr.Q(a, complete = TRUE)[, (u + 1):p]
-      objfun <- - n/2 * (sum(log(e1$values)) + sum(log(e2$values)) + sum(log(e3$values))) + e5
+      objfun <- - n/2 * (sum(log(e1$values)) + sum(log(e2$values))) + sum(log(e3$values)) + e5
       Gammahat <- as.matrix(Gammahat)
       Gamma0hat <- as.matrix(Gamma0hat)
     
